import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.process.ImageProcessor;
import imagescience.image.Dimensions;
import imagescience.image.Image;
import imagescience.utility.I5DResource;
import imagescience.utility.ImageScience;
import java.awt.image.ColorModel;

public final class TJ {

    private static final String NAME = "TransformJ";
    private static final String VERSION = "2.7.0";
    private static final String MINIJVERSION = "1.38u";
    private static final String MINISVERSION = "2.1.0";

    public static String name() { return NAME; }

    public static String version() { return VERSION; }

    static boolean libcheck() {

	if (IJ.getVersion().compareTo(MINIJVERSION) < 0) {
	    error("This plugin requires ImageJ version "+MINIJVERSION+" or higher");
	    return false;
	}

	try {
	    if (ImageScience.version().compareTo(MINISVERSION) < 0)
		throw new IllegalStateException();

	} catch (Throwable e) {
	    error("This plugin requires ImageScience version "+MINISVERSION+" or higher");
	    return false;
	}

	return true;
    }

    static ImagePlus imageplus() {

	final ImagePlus imp = WindowManager.getCurrentImage();
	if (imp == null)  {
	    error("There are no images open");
	    return null;
	}

	return imp;
    }

    static void show(final Image newimg, final ImagePlus inimp) {

	ImagePlus newimp = newimg.imageplus();
	newimp.setCalibration(inimp.getCalibration().copy());
	final ImageProcessor ip = inimp.getProcessor();
	final ImageProcessor newip = newimp.getProcessor();
	final ColorModel cm = ip.getColorModel();
	newip.setColorModel(cm);
	newimp.getStack().setColorModel(cm);
	final double[] minmax = newimg.extrema();
	if (TJ_Options.adopt) newip.setMinAndMax(ip.getMin(),ip.getMax());
	else newip.setMinAndMax(minmax[0],minmax[1]);
	boolean i5dinstalled = false;
	try { Class.forName("i5d.Image5D"); i5dinstalled = true; } catch (Exception e) { }
	if (i5dinstalled) {
	    final Dimensions newdims = newimg.dimensions();
	    if (newdims.t > 1 || newdims.s > 1) {
		newimp = I5DResource.convert(newimp,true);
		if (I5DResource.instance(inimp)) {
		    I5DResource.transfer(inimp,newimp);
		    if (!TJ_Options.adopt)
			I5DResource.minmax(newimp,minmax[0],minmax[1]);
		}
	    }
	}
	newimp.changes = TJ_Options.asksave;
	newimp.show();
    }

    static void error(final String message) {

	IJ.showMessage(NAME+": Error",message+".");
	IJ.showProgress(1.0);
	IJ.showStatus("");
    }

    static void debug(final String message) {

	if (TJ_Options.debug) IJ.log(message);
    }

}
